SystemOrganization addCategory: #'GrandMa-Worker'!

Error subclass: #GMMalformedServerResponseException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GrandMa-Worker'!

!GMMalformedServerResponseException commentStamp: '<historical>' prior: 0!
This error is raised when the server answers something that is not understood by the worker.!

Error subclass: #GMNoTaskOnServerException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GrandMa-Worker'!

!GMNoTaskOnServerException commentStamp: '<historical>' prior: 0!
This error is raised when the worker tries to get a task from the server but the server does not have any tasks.!

Error subclass: #GMServerError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GrandMa-Worker'!

!GMServerError commentStamp: '<historical>' prior: 0!
This error is raised when the worker cannot connect to the server.!

Object subclass: #GMWorker
	instanceVariableNames: 'serverAddress isRunning'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GrandMa-Worker'!

!GMWorker commentStamp: '<historical>' prior: 0!
A worker polls a GMServer to check if there is a task available. 

serverAddress should not have a trailing slash (default: localhost:7070).

It uses OSProcess to execute it.

Usage:
worker := GMWorker new.
worker serverAddress: 'http://grandma.com'
worker start.
worker stop.!

!GMWorker methodsFor: 'polling' stamp: 'FrancoisStephany 4/15/2012 16:04'!
executeTask: aTaskDictionary
	| env path pipe |
	env := OSProcess thisOSProcess environment copy.
	path := (env at: #PATH), ':/usr/local/bin/'.
	env at: #PATH put: path.
	
	OSProcess thisOSProcess discardExitedChildren.
	OSProcess thisOSProcess resetChildProcessDictionary.
	
	pipe := (PipeableOSProcess command: (aTaskDictionary at: 'command') environment: env) output.
	pipe closePipes
! !

!GMWorker methodsFor: 'accessing' stamp: 'FrancoisStephany 4/15/2012 14:59'!
failureUrl
	^self serverAddress , '/failure'.! !

!GMWorker methodsFor: 'polling' stamp: 'FrancoisStephany 4/15/2012 15:46'!
fetchTask
	| response json |
	response := ZnEasy post: self popUrl data: (ZnEntity text: '').
	response isSuccess
		ifFalse: [ GMServerError signal ].
	response isNoContent
		ifTrue:  [ GMNoTaskOnServerException signal ]
		ifFalse: [json := Json readFrom: response contents readStream ].
	
	((json includesKey: 'uuid') and: [ json includesKey: 'command' ])
		ifTrue: [ ^json ]
		ifFalse: [ GMMalformedServerResponseException signal ].
	! !

!GMWorker methodsFor: 'initialize-release' stamp: 'FrancoisStephany 4/15/2012 15:11'!
initialize
	super initialize.
	isRunning := false.
	self startLoop.! !

!GMWorker methodsFor: 'accessing' stamp: 'FrancoisStephany 4/15/2012 15:12'!
isRunning
	^isRunning! !

!GMWorker methodsFor: 'accessing' stamp: 'FrancoisStephany 4/15/2012 15:22'!
popUrl
	^self serverAddress , '/pop'.! !

!GMWorker methodsFor: 'accessing' stamp: 'FrancoisStephany 4/15/2012 14:51'!
serverAddress
	^ serverAddress! !

!GMWorker methodsFor: 'accessing' stamp: 'FrancoisStephany 4/15/2012 14:51'!
serverAddress: aString
	serverAddress := aString! !

!GMWorker methodsFor: 'polling' stamp: 'FrancoisStephany 4/15/2012 15:09'!
start
	isRunning := true.! !

!GMWorker methodsFor: 'polling' stamp: 'FrancoisStephany 4/15/2012 16:10'!
startLoop
	| task result |
	[ self isRunning ] whileTrue: [
		task := self fetchTask.
		task isNil
			ifTrue: [ Delay forSeconds: 120 ]
			ifFalse: [ 
				result := self executeTask: task.
				self answerResult: result ]].! !

!GMWorker methodsFor: 'polling' stamp: 'FrancoisStephany 4/15/2012 15:09'!
stop
	isRunning := false.! !

!GMWorker methodsFor: 'accessing' stamp: 'FrancoisStephany 4/15/2012 14:59'!
successUrl
	^self serverAddress , '/success'.! !
